#include <PalmOS.h>
#include <SoundMgr.h>


typedef struct{
  Int16 width;
  Int16 height;
  UInt16 rowBytes;
  UInt16 /*BitmapFlagsType*/ flags;
  UInt8 pixelSize;
  UInt8 version;
} MyBitmapType;


typedef struct{
	Boolean set;
	UInt16 hour;
	UInt16 minute;
	LocalID LID;
	UInt32 recID;
	char  name[32];
}alarmInfoType;

typedef struct{
	SndMidiListItemType* list;
	UInt16 selItem;
}fakeListDrawCallbackData;

typedef struct{
	alarmInfoType* alarm;
	Boolean preview;
}playSoundCallbackData;



#define timeX 0
#define timeY 0

#define time2X 100
#define time2Y 30

#define dateX 80
#define dateY 145

#define dayX 5
#define dayY 145

#define START_SECS_PER_PIC 20


typedef UInt8 bool;


typedef Boolean (*dialogCallback)(char* text,char* op1,char* op2, char* op3, char* op4,void* param);
									//return if all handled


void drawCenteredString(char* s,UInt16 width,UInt16 x, UInt16 y){
	
	Int16 L=StrLen(s);
	Int16 W=FntCharsWidth(s,L);
	
	if(W<=width){
		WinDrawChars(s,L,x+(width-W)/2,y);
	}
	else{
		WinDrawTruncChars(s,L,x,y,width);
	}
	
	
}


void waitKeyRelease(){
	while(KeyCurrentState())
		SysTaskDelay(5);
}

UInt16 dialog(UInt32 timeout,Boolean saveBehind,char* text,char* op1,char* op2, char* op3, char* op4, dialogCallback callback,void* callbackParam){
	
	RectangleType r;
	Err e;
	WinHandle savedBits;
	UInt32 mask,k;
	FontID font;
	UInt32 time;
	
	
	RctSetRectangle(&r,0,0,160,160);
	
	
	
	//save current environment
	if(saveBehind){
		savedBits=WinSaveBits(&r,&e);
		
		if(!savedBits)
			SysFatalAlert("WinSaveBits failed");
	}
	
	font=FntSetFont(stdFont);
	
	
	//clear window
	
	WinEraseRectangle(&r,0);
	
	
	
	
	
	
	
	{
		//word wrap text;
		UInt16 L,s=0,dL;
		UInt16 y=30;
		char* txt=text;
		
		while(StrLen(text)>s+1){
			L=FntWordWrap(txt,150);
			
			
			dL=L;
			if(txt[L-1]=='\n')
				dL--;
				
			WinDrawChars(txt,dL,4,y);
			y+=12;
			txt+=L;
			s+=L;
		}
	}
		
	
	
	//drawing done now we draw options
	//4 1-pixel colomns are saved for separators so we have 156 pixels to manage
	//so about 39 pixels per option;
	
	
	//first we draw separators
	
	WinDrawLine(0,147,159,147);		//horiz
	WinDrawLine(40,147,40,159);		//vert 1
	WinDrawLine(79,147,79,159);		//vert 2
	WinDrawLine(80,147,80,159);		//vert 3
	WinDrawLine(119,147,119,159);	//vert 4
	
	
	//now draw texts and simultaneously make key mask
	mask=0;
	
	if(op1){
		drawCenteredString(op1,39,1,148);
		mask|=0x00000008;
	}
	
	if(op2){
		drawCenteredString(op2,39,41,148);
		mask|=0x00000010;
	}
	
	if(op3){
		drawCenteredString(op3,39,81,148);
		mask|=0x00000020;
	}
	
	if(op4){
		drawCenteredString(op4,39,120,148);
		mask|=0x00000040;
	}
	
	
	
	//call callback
	
	if(callback)
		
		callback(text,op1,op2,op3,op4,callbackParam);
	
	
	
	
	//now cycle and wait for reply...
	
	time=TimGetTicks();
	
	while((k=(KeyCurrentState()&mask))==0 && time+timeout>TimGetTicks()){
		SysTaskDelay(10);
	}
	
	waitKeyRelease();
	
	
	//now find what was pressed
	
	if(k==0)
		k=5;
	
	else if(k&0x00000008)
		k=0;
	
	else if(k&0x00000010)
		k=1;
	
	else if(k&0x00000020)
		k=2;
	
	else if(k&0x00000040)
		k=3;
	
	
	//restore current environment
	
	if(saveBehind)
		WinRestoreBits(savedBits,0,0);
	
	
	FntSetFont(font);
	
	
	
	//return the pressed key
	
	
	
	return k;
}




void drawBitmap(UInt16 resID,UInt16 x,UInt16 y){
	MemHandle mh;
	
	
	mh=DmGetResource('Tbmp',resID);
	if(mh){
		WinDrawBitmap(MemHandleLock(mh),x,y);
		MemHandleUnlock(mh);
		DmReleaseResource(mh);
	}
	else{
		char x[32];
		StrIToA(x,resID);
		ErrAlertCustom(0,x,"Bitmap ID "," not found");
	}
	
}

void drawChromeString(char* str,UInt16 xp,UInt16 yp){
	UInt16 x=xp;
	UInt16 i;
	char c;
	
	
	for(i=0;i<StrLen(str);i++){
		
		c=str[i];
		if(c>='0' && c<='9')
			drawBitmap(1000+c-'0',x,yp);
			
		if(c==':')
			drawBitmap(1100,x,yp);
		
		if(c=='a' || c=='A')
			drawBitmap(1200,x,yp+8);
		if(c=='p' || c=='P')
			drawBitmap(1201,x,yp+8);
		
		if(c=='m' || c=='M')
			drawBitmap(1300,x,yp+8);
		
		
		x+=27;
	}
}

void DrawCoolString(char* c,UInt16 len,UInt16 x,UInt16 y){
	
	Int16 a,b;
	
	
	WinSetDrawMode(winMask);
	WinSetTextColor(255);
	
	for(a=-1;a<=1;a++){
		for(b=-1;b<=1;b++){
			WinPaintChars(c,len,x+a,y+b);
		}
	}
	
	WinSetDrawMode(winInvert);
	
	WinSetTextColor(255);
	
	WinPaintChars(c,len,x,y);
	
}

void MyBmpGetDimensions(BitmapPtr bitmapPtr,Coord *widthP,Coord *heightP,UInt16 *rowBytesP){
	
	
	MyBitmapType* bitmapP=(MyBitmapType*)(bitmapPtr);
	
	if(!bitmapP)
		return;
	
	
	if(widthP)
		*widthP=bitmapP->width;
	
	if(heightP)
		*heightP=bitmapP->height;
	
	if(rowBytesP)
		*rowBytesP=bitmapP->rowBytes;
	
}

void drawBitmapHalfSize(BitmapPtr bmp,UInt16 x, UInt16 y){		//asumes a lot of things
	
	
	UInt16 w,h,sBytesPerRow,dBytesPerRow,a,b,sRO,dRO;
	Err e;
	UInt16* sBits;
	UInt16* dBits;
	WinHandle wh;
	BitmapPtr smallBmp;
	WinHandle bWh;
	FontID font;
	RectangleType r;
	
	
	
	MyBmpGetDimensions(bmp,(Int16*)&w,(Int16*)&h,NULL);
	
	RctSetRectangle(&r,x,y,w/2,h/2);
	
	WinEraseRectangle(&r,0);
	
	font=FntSetFont(stdFont);
	
	WinDrawChars("Generating",10,x,y);
	WinDrawChars("Preview...",10,x,y+12);
	
	FntSetFont(font);
	
		
	
	
	bWh=WinCreateOffscreenWindow(w,h,screenFormat,&e);
	
	if(!bWh)
		SysFatalAlert("Error creating bigBitmap window");
	
	
	wh=WinCreateOffscreenWindow(w/2,h/2,screenFormat,&e);
	
	if(!wh)
		SysFatalAlert("Error creating smallBitmap window");
	
	
	bWh=WinSetDrawWindow(bWh);
	WinDrawBitmap(bmp,0,0);
	bWh=WinSetDrawWindow(bWh);
	bmp=WinGetBitmap(bWh);
	
	MyBmpGetDimensions(bmp,NULL,NULL,&sBytesPerRow);
	
	
	smallBmp=WinGetBitmap(wh);
	
	
	MyBmpGetDimensions(smallBmp,NULL,NULL,&dBytesPerRow);
	
	sBits=BmpGetBits(bmp);
	dBits=BmpGetBits(smallBmp);
	
	dBytesPerRow/=2;
	
	h/=2;
	w/=2;
	
	
	sRO=0;
	dRO=0;
	
	for(a=0;a<h;a++){		//row
		
		for(b=0;b<w;b++){	//col
			
			dBits[dRO+b]=sBits[sRO+(b<<1)];
			
		}
		
		sRO+=sBytesPerRow;
		dRO+=dBytesPerRow;
	}
	
	WinDrawBitmap(smallBmp,x,y);
	
	WinDeleteWindow(wh,false);
	
	WinDeleteWindow(bWh,false);
}

char* monthToString(UInt16 weekDay){
	switch(weekDay){
		case 1:
			return "Jan";

		case 2:
			return "Feb";

		case 3:
			return "Mar";

		case 4:
			return "Apr";

		case 5:
			return "May";

		case 6:
			return "Jun";

		case 7:
			return "Jul";

		case 8:
			return "Aug";

		case 9:
			return "Sep";

		case 10:
			return "Oct";

		case 11:
			return "Nov";

		case 12:
			return "Dec";

	}
	return NULL;
}

char* weekDayToString(UInt16 month){
	switch(month){
		case 0:
			return "Sunday";

		case 1:
			return "Monday";

		case 2:
			return "Tuesday";

		case 3:
			return "Wednesday";

		case 4:
			return "Thursday";

		case 5:
			return "Friday";

		case 6:
			return "Saturday";

	}
	
	return NULL;
}

void dateTimeFromSecs(UInt32 sec,char* date,char* time,char* Ampm,char* day){
	DateTimeType dt;
	char x[32];
	
	TimSecondsToDateTime(sec,&dt);
	
	if(date)
		DateToAscii(dt.month,dt.day,dt.year,dfMDYLongWithComma,date); 
	
	StrIToA(x,sec%60);
	if(StrLen(x)==1){
		x[2]=0;
		x[1]=x[0];
		x[0]='0';
	}
	
	if(time)
		TimeToAscii(dt.hour,dt.minute,tfColon,time); 
	

	if(Ampm){
		TimeToAscii(dt.hour,dt.minute,tfHoursAMPM,x); 
		MemMove(x,&x[StrLen(x)-2],3);
		StrCopy(Ampm,x);
	}
	
	if(day)
		StrCopy(day,weekDayToString(dt.weekDay));

}


UInt16 enumerateBitmaps(BitmapPtr** bP,LocalID** LIDsP){
	UInt16 num=0,i;
	BitmapPtr* b;
	MemHandle dbsH;
	SysDBListItemType* dbs;
	DmOpenRef db;
	LocalID* LIDs;
	
	SysCreateDataBaseList('cBmp','PeCL',&num,&dbsH,false);
	if(dbsH){
		
		b=   MemPtrNew(num*sizeof(BitmapPtr));
		LIDs=MemPtrNew(num*sizeof(LocalID));
		   *bP=b;
		*LIDsP=LIDs;
		
		dbs=MemHandleLock(dbsH);
		for(i=0;i<num;i++){
			
			LIDs[i]=dbs[i].dbID;
			
			//make database backupable
			{
				UInt16 attr;
				
				DmDatabaseInfo(0,dbs[i].dbID,NULL,&attr,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
				
				attr|=dmHdrAttrBackup;
				
				DmSetDatabaseInfo(0,dbs[i].dbID,NULL,&attr,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);

			}
			
			
			db=DmOpenDatabase(0,dbs[i].dbID,dmModeReadOnly);
			b[i]=MemHandleLock(DmGet1Resource('Tbmp',0));
			DmCloseDatabase(db);
				
		}
		MemHandleUnlock(dbsH);
		MemHandleFree(dbsH);
	}
	else
		num=0;
	
	
	
	return num;
	
}

void releaseBitmaps(BitmapPtr* b,LocalID* LIDs,UInt16 num){
	UInt16 i;
	MemHandle mh;
	
	
	for(i=0;i<num;i++){
		
		mh=MemPtrRecoverHandle(b[i]);
		MemHandleUnlock(mh);
		DmReleaseResource(mh);
		
	}
	
	MemPtrFree(b);
	MemPtrFree(LIDs);
	
}

void SndComplFunc(void *channelP,UInt32 userData){
	//nothign
}

Boolean SndBlockingFunc(void *channelP,UInt32 userData,Int32 time){
	return KeyCurrentState()==0;
}

Boolean playSoundCallback(char* text,char* op1,char* op2, char* op3, char* op4,void* data){
	
	playSoundCallbackData* cd=(playSoundCallbackData*)data;
	alarmInfoType* alarm=cd->alarm;
	DmOpenRef db;
	UInt8* midiData;
	MemHandle mh;
	UInt16 id;
	Err e;
	
	db=DmOpenDatabase(0,alarm->LID,dmModeReadOnly);
	if(db){
	
		if(!DmFindRecordByID(db,alarm->recID,&id)){
		
			mh=DmGetRecord(db,id);
			if(mh){
				
				midiData=MemHandleLock(mh);
				
				do{
					SndSmfCallbacksType cb={{&SndComplFunc,0},{&SndBlockingFunc},0,0};
					SndSmfOptionsType opts={0,sndSmfPlayAllMilliSec,sndMaxAmp,false,0,0};
					
					if(cd->preview)
						opts.amplitude=10;
					
					e=SndPlaySmf(NULL,1/*SndSmfCmdPlay*/,midiData,&opts,NULL,&cb,0);
				}while(e!=sndErrInterrupted);
				
				MemHandleUnlock(mh);
				DmReleaseRecord(db,id,false);
			}
		}
		
		DmCloseDatabase(db);
	}
	return false;
}

/*
typedef struct{
	Boolean set;
	UInt16 hour;
	UInt16 minute;
	LocalID LID;
	UInt16 recID;
	char  name[32];
}alarmInfoType;


typedef struct{
	SndMidiListItemType* list;
	UInt16 selItem;
}fakeListDrawCallbackData;

*/

Boolean selectAlarmCallback(char* text,char* op1,char* op2, char* op3, char* op4,void* data){
	
	fakeListDrawCallbackData* d=(fakeListDrawCallbackData*)data;
	
	//draw list
	
	
	WinDrawChars(d->list[d->selItem].name,StrLen(d->list[d->selItem].name),10,50);
	
	
	return false;
}


void setAlarmSound(alarmInfoType* alarm){
	UInt16 num;
	MemHandle listH;
	SndMidiListItemType* items;
	UInt16 ret,selItem=0;
	fakeListDrawCallbackData d;

	
	SndCreateMidiList('psys',true,&num,&listH);
	
	
	items=MemHandleLock(listH);
	
	
	do{
		
		d.list=items;
		d.selItem=selItem;
		
		ret=dialog(0x0FFFFFFF,false,"SELECT ALARM SOUND:","OK","Play","Down","Up",&selectAlarmCallback,&d);
			
		switch(ret){
			
			case 0:		//ok
				
				alarm->LID=items[selItem].dbID;
				alarm->recID=items[selItem].uniqueRecID;
				StrCopy(alarm->name,items[selItem].name);
				
				break;
			
			case 1:		//play
			{
				alarmInfoType a;
				playSoundCallbackData d;
				
				d.alarm=&a;
				d.preview=true;
				
				a.LID=items[selItem].dbID;
				a.recID=items[selItem].uniqueRecID;
				
				dialog(0x0FFFFFFF,false,"Sound preview","OK",NULL,NULL,NULL,&playSoundCallback,&d);
				
				break;
			}
			case 2:		//Down
				
				selItem++;
				selItem%=num;
				
				break;
			
			
			case 3:		//Up
				
				selItem--;
				selItem+=num;
				selItem%=num;
				
				break;
			
			
		}
		
	}while(ret!=0);
	
	MemHandleUnlock(listH);
	MemHandleFree(listH);
	
}

#define HIGHLITED_ITEM_MAX_FOR_SET_TIME 1	/* hours=0, minute=1*/

void setAlarmTime(UInt16* hourP,UInt16* minuteP){		//return if set
	char msg[200];
	char n[12];
	UInt16 ret;
	UInt16 higlightedItem=0;
	UInt16 hour,minute;
	
	
	hour=*hourP;
	minute=*minuteP;
	
	
	while(1){	//will exit with other means...
		
	
		StrCopy(msg,"Alrm time is:\n");
		StrIToA(n,hour%12==0?12:hour%12);
		
		if(higlightedItem==0) StrCat(msg," >>");
		StrCat(msg,n);
		if(higlightedItem==0) StrCat(msg,"<< ");
		
		StrCat(msg,":");
		
		if(minute<10){
			StrIToA(&n[1],minute);
			n[0]='0';
		}
		else{
			StrIToA(n,minute);
		}
		
		if(higlightedItem==1) StrCat(msg," >>");
		StrCat(msg,n);
		if(higlightedItem==1) StrCat(msg,"<< ");
		
		StrCat(msg," ");
		StrCat(msg,hour>=12?"PM":"AM");
		StrCat(msg,".");
		
		
		ret=dialog(0x0FFFFFFF,false,msg,"OK","Next","+1","-1",NULL,NULL);
		
		switch(ret){
			
			case 0:		//ok
				*hourP=hour;
				*minuteP=minute;
				return;
			
			case 1:		//next item
				higlightedItem=(higlightedItem+1)%(HIGHLITED_ITEM_MAX_FOR_SET_TIME+1);
				break;
			
			case 2:		//+1
				
				if(higlightedItem==0){		//hours
					
					hour=(hour+1)%24;
					
				}
				if(higlightedItem==1){		//minutes
					
					minute=(minute+1)%60;
					
				}
				break;
			
			case 3:		//-1
				
				if(higlightedItem==0){		//hours
					
					hour=(hour-1+24)%24;
					
				}
				if(higlightedItem==1){		//minutes
					
					minute=(minute-1+60)%60;
					
				}
				break;
			
			case 5:	//just timeout
				break;
			
			
		}
	}
	*hourP=hour;
	*minuteP=minute;
}


void setAlarm(alarmInfoType* alarm){
	
	UInt16 ret;
	char msg[128];
	char n[32];
	
	
	while(1){	//will exit with other means...
		
		
		StrCopy(msg,"ALARM: Currently ");
		
		if(alarm->set){
			
			StrCat(msg,"set to play sound\"");
			StrCat(msg,alarm->name);
			StrCat(msg,"\" at");
			StrIToA(n,alarm->hour%12==0?12:alarm->hour%12);
			StrCat(msg,n);
			StrCat(msg,":");
			
			if(alarm->minute<10){
				StrIToA(&n[1],alarm->minute);
				n[0]='0';
			}
			else{
				StrIToA(n,alarm->minute);
			}
			
			StrCat(msg,n);
			StrCat(msg," ");
			StrCat(msg,alarm->hour>=12?"PM":"AM");
			
			
		}
		else{
			StrCat(msg,"not set");
		}
		
		
		ret=dialog(0x0FFFFFFF,false,msg,"Done",alarm->set?"Remove":NULL,"Time","Sound",NULL,NULL);
		
		switch(ret){
			
			case 0:		//ok
				if((!alarm->LID || !alarm->name) && alarm->set){
					
					if(dialog(0x0FFFFFFF,false,"Alarm will not be set as no sound has been defined. Exit anyways","No","Yes",NULL,NULL,NULL,NULL))
						return;
					
				}
				else
					return;
				
				break;
			
			case 1:		//Remove
				alarm->set=false;
			
			case 2:		//Time
				setAlarmTime(&alarm->hour,&alarm->minute);
				alarm->set=true;
				break;
			
			case 3:		//Sound
				setAlarmSound(alarm);
				break;
			
		}
			
	}
}





#define HIGHLITED_ITEM_MAX_FOR_SET_DATA 2 	/* day=0, month=1, year=2*/


char* weekDayToString(UInt16 weekDay);
char* monthToString(UInt16 month);

void setDate(){
	char msg[200];
	char n[12];
	UInt16 ret;
	UInt16 higlightedItem=0;
	DateTimeType dt;
	
	
	
	while(1){	//will exit with other means...
		
		TimSecondsToDateTime(TimGetSeconds(),&dt);
	
		StrCopy(msg,"Current date is:\n");
		StrCat(msg,weekDayToString(dt.weekDay));
		StrCat(msg,", ");
		
		if(higlightedItem==0) StrCat(msg," >>");
		StrCat(msg,monthToString(dt.month));
		if(higlightedItem==0) StrCat(msg,"<< ");
		
		StrCat(msg," ");
		StrIToA(n,dt.day);
		
		if(higlightedItem==1) StrCat(msg," >>");
		StrCat(msg,n);
		if(higlightedItem==1) StrCat(msg,"<< ");
		
		
		StrCat(msg," ");
		StrIToA(n,dt.year);
		
		if(higlightedItem==2) StrCat(msg," >>");
		StrCat(msg,n);
		if(higlightedItem==2) StrCat(msg,"<< ");
		
		StrCat(msg,".");
		
		
		ret=dialog(500/*5 sec*/,false,msg,"OK","Next","+1","-1",NULL,NULL);
		
		switch(ret){
			
			case 0:		//ok
				return;
			
			case 1:		//next item
				higlightedItem=(higlightedItem+1)%(HIGHLITED_ITEM_MAX_FOR_SET_DATA+1);
				break;
			
			case 2:		//+1
				
				if(higlightedItem==0){		//month
					
					dt.month++;
					if(dt.month>12)
						dt.month-=12;
					
				}
				if(higlightedItem==1){		//day
					UInt16 daysInCurrMonth;
					
					daysInCurrMonth=DaysInMonth(dt.month,dt.year);			
					
					dt.day++;
					if(dt.day>daysInCurrMonth)
						dt.day-=daysInCurrMonth;
					
				}
				if(higlightedItem==2){		//year
					
					dt.year++;
					
				}
				break;
			
			case 3:		//-1
				if(higlightedItem==0){		//month
					
					dt.month=dt.month-1+12;
					if(dt.month>12)
						dt.month-=12;
					
				}
				if(higlightedItem==1){		//day
					UInt16 daysInCurrMonth;
					
					daysInCurrMonth=DaysInMonth(dt.month,dt.year);			
					
					dt.day=dt.day-1+daysInCurrMonth;
					if(dt.day>daysInCurrMonth)
						dt.day=1;
					
				}
				if(higlightedItem==2){		//year
					
					dt.year--;
					
				}
				break;

			
			case 5:	//just timeout
				break;
			
			
		}
		
		TimSetSeconds(TimDateTimeToSeconds(&dt));
		
	}
}


void setTime(){
	char msg[200];
	char n[12];
	UInt16 ret;
	UInt16 higlightedItem=0;
	DateTimeType dt;
	
	
	
	while(1){	//will exit with other means...
		
		TimSecondsToDateTime(TimGetSeconds(),&dt);
	
		StrCopy(msg,"Current time is:\n");
		StrIToA(n,dt.hour%12==0?12:dt.hour%12);
		
		if(higlightedItem==0) StrCat(msg," >>");
		StrCat(msg,n);
		if(higlightedItem==0) StrCat(msg,"<< ");
		
		StrCat(msg,":");
		
		if(dt.minute<10){
			StrIToA(&n[1],dt.minute);
			n[0]='0';
		}
		else{
			StrIToA(n,dt.minute);
		}
		
		if(higlightedItem==1) StrCat(msg," >>");
		StrCat(msg,n);
		if(higlightedItem==1) StrCat(msg,"<< ");
		
		StrCat(msg," ");
		StrCat(msg,dt.hour>=12?"PM":"AM");
		StrCat(msg,".");
		
		
		ret=dialog(500/*5 sec*/,false,msg,"OK","Next","+1","-1",NULL,NULL);
		
		switch(ret){
			
			case 0:		//ok
				return;
			
			case 1:		//next item
				higlightedItem=(higlightedItem+1)%(HIGHLITED_ITEM_MAX_FOR_SET_TIME+1);
				break;
			
			case 2:		//+1
				
				if(higlightedItem==0){		//hours
					
					dt.hour=(dt.hour+1)%24;
					
				}
				if(higlightedItem==1){		//minutes
					
					dt.minute=(dt.minute+1)%60;
					
				}
				break;
			
			case 3:		//-1
				
				if(higlightedItem==0){		//hours
					
					dt.hour=(dt.hour-1+24)%24;
					
				}
				if(higlightedItem==1){		//minutes
					
					dt.minute=(dt.minute-1+60)%60;
					
				}
				break;
			
			case 5:	//just timeout
				break;
			
			
		}
		
		TimSetSeconds(TimDateTimeToSeconds(&dt));
		
	}
}


void dateTimeSet(){
	char msg[200];
	UInt16 ret;
	char date[32];
	char time[32];
	char Ampm[12];
	char day[32];
	
	
	
	while(1){	//will exit with other means...
		
		StrCopy(msg,"Now is ");
		dateTimeFromSecs(TimGetSeconds(),date,time,Ampm,day);
		StrCat(msg,day);
		StrCat(msg," ");
		StrCat(msg,date);
		StrCat(msg," ");
		StrCat(msg,time);
		StrCat(msg," ");
		StrCat(msg,Ampm);
		
		ret=dialog(1000/*10 sec*/,false,msg,"OK",NULL,"Set date","Set time",NULL,NULL);
		
		switch(ret){
			
			case 0:		//ok
				return;
			
			
			case 1:		//impossible
				return;
				
				
			case 2:		//date
				setDate();
				break;
			
			
			case 3:		//time
				setTime();
				break;
			
			case 5:		//just timeout
				break;

		}
	}
}

void picDelaySet(UInt32* secsPerPic){
	
	char msg[140];
	char n[12];
	UInt16 ret;
	
	
	
	while(1){	//will exit with other means...
		
		StrCopy(msg,"This is the time (in seconds) that every picture is displayed. Current value is ");
		StrIToA(n,*secsPerPic);
		StrCat(msg,n);
		StrCat(msg," seconds. Minumum is 5 seconds, maximum is 60.");
		
		
		
		ret=dialog(0x0FFFFFFF,false,msg,"OK","+1","+5","-5",NULL,NULL);
		
		
		switch(ret){
			
			case 0:		//ok
				return;
				
			case 1:		//+1
				if(*secsPerPic<=59)
					*secsPerPic+=1;
				
				break;
				
			
			case 2:		//+5
				if(*secsPerPic<=55)
					*secsPerPic+=5;
				
				break;
				
				
			case 3:		//-5
				if(*secsPerPic>=10)
					*secsPerPic-=5;
				
				break;
			
		}
	}
}

Boolean bitmapDrawCallback(char* text,char* op1,char* op2, char* op3, char* op4,void* data){
	drawBitmapHalfSize((BitmapPtr)data,40,60);
	return false;
}

void managePics(UInt16* numBitmaps,BitmapPtr* b,LocalID* LIDs){
	UInt16 ret;
	UInt16 selItem=0;
	char msg[128];
	char n[12];
	
	
	
	while(1){		//will exit with other means...
		
		
		StrCopy(msg,"Number of bitmaps found: ");
		StrIToA(n,*numBitmaps);
		StrCat(msg,n);
		StrCat(msg,".\nCurrent bitmap is number ");
		StrIToA(n,selItem+1);
		StrCat(msg,n);
		StrCat(msg,".");
		
		
		
		ret=dialog(0x0FFFFFF,false,msg,"OK","Prev","Next","Delete",&bitmapDrawCallback,b[selItem]);
		
		
		switch(ret){
			
			case 0:		//return
				return;
			
			
			case 1:		//prev
				selItem=(selItem+(*numBitmaps)-1)%(*numBitmaps);
				break;
			
			
			case 2:		//next
				selItem=(selItem+1)%(*numBitmaps);
				break;
			
			
			case 3:		//del
				if(*numBitmaps==1){
					dialog(0x0FFFFFF,false,"Cannot delete the only picture","OK",NULL,NULL,NULL,NULL,NULL);
				}
				else{
					
					MemHandle mh;
					
					mh=MemPtrRecoverHandle(b[selItem]);
					MemHandleUnlock(mh);
					DmReleaseResource(mh);
					
					if(DmDeleteDatabase(0,LIDs[selItem])){
						
						dialog(0x0FFFFFF,false,"Error deleting","OK",NULL,NULL,NULL,NULL,NULL);
						
						
						b[selItem]=MemHandleLock(mh);
						
						
					}
					else{
						
						for(ret=selItem;ret<(*numBitmaps)-1;ret++){
							   b[ret]=   b[ret+1];
							LIDs[ret]=LIDs[ret+1];
						}
						
						(*numBitmaps)--;
						
						
						if(selItem==(*numBitmaps))
							selItem--;
						
					}
				}
				break;
			
		}

		
		
	}
}

void pics(UInt32* secsPerPic,UInt16* numBitmaps,BitmapPtr* b,LocalID* LIDs){
	UInt16 ret;
	
	while(1){		//will exit with other means...
		
		ret=dialog(0x0FFFFFF,false,"PICS","Return",NULL,"Pics...","Pic delay",NULL,NULL);
		
		switch(ret){
			
			case 0:		//return
				return;
			
			
			case 1:		//null
				break;
			
			
			case 2:		//manage pics
				managePics(numBitmaps,b,LIDs);
				break;
			
			
			case 3:		//about
				picDelaySet(secsPerPic);
				break;
			
		}
	}
}


void settings(UInt32* secsPerPic,UInt16* numBitmaps,BitmapPtr* b,LocalID* LIDs,alarmInfoType* alarm){
	UInt16 ret;
	
	while(1){		//will exit with other means...
		
		ret=dialog(0x0FFFFFF,false,"SETTINGS","Return","Set time","Pics...","Alarm",NULL,NULL);
		
		switch(ret){
			
			case 0:		//return
				return;
			
			
			case 1:		//Set time
				dateTimeSet();
				break;
			
			
			case 2:		//pics
				pics(secsPerPic,numBitmaps,b,LIDs);
				break;
			
			
			case 3:		//alarm
				setAlarm(alarm);
				break;
			
		}
	}
}


void menu(Boolean* exit,UInt32* secsPerPic,UInt16* numBitmaps,BitmapPtr* b,LocalID* LIDs,alarmInfoType* alarm){
	
	UInt16 ret;
	
	while(1){		//will exit with other means...
		
		ret=dialog(0x0FFFFFFF,false,"MAIN MENU","Return","Exit app","Settings","About",NULL,NULL);
		
		switch(ret){
			
			case 0:		//return
				return;
			
			
			case 1:		//exit app
				*exit=true;
				return;
			
			
			case 2:		//Settings
				settings(secsPerPic,numBitmaps,b,LIDs,alarm);
				break;
			
			
			case 3:		//about
				dialog(0x0FFFFFFF,false,"ABOUT: Permaclock is just a simple app to make use of a palm with a broken digitizer as a simple digital picture frame/clock. Freeware (c) 2005 Palm Powerups.","OK",NULL,NULL,NULL,NULL,NULL);
				break;
			
		}
	}
}

void processCheckAlarm(alarmInfoType* alarm){
	
	DateTimeType dt;
	char msg[128];
	char n[32];
	UInt16 ret;
	playSoundCallbackData cd;

	
	if(!alarm->set)
		return;
	
	
	cd.alarm=alarm;
	cd.preview=false;
	
	
	TimSecondsToDateTime(TimGetSeconds(),&dt);
	
	
	if(dt.hour==alarm->hour && dt.minute==alarm->minute){		//triggered
		
		
		StrCopy(msg,"ALARM. It is ");
		StrIToA(n,dt.hour%12==0?12:dt.hour%12);
		StrCat(msg,n);
		StrCat(msg,":");
		if(dt.minute<10){
			StrIToA(&n[1],dt.minute);
			n[0]='0';
		}
		else{
			StrIToA(n,dt.minute);
		}
		StrCat(msg,n);
		StrCat(msg," !");
		
		
		
		ret=dialog(0x0FFFFFFF,false,msg,"Dissmiss","Snooze",NULL,NULL,&playSoundCallback,&cd);
		
		if(ret==0){
			alarm->set=0;
			
		}
		else{
			
			alarm->minute+=5;
			if(alarm->minute>=60){
				alarm->minute-=60;
				alarm->hour++;
			}
			alarm->hour%=24;
		}
		
		
	}
	
	
	
}


void SysTaskDelayNoKeys(UInt32 ticks){
	UInt32 oldTicks=TimGetTicks();
	
	
	while((KeyCurrentState()==0) && (TimGetTicks()<=oldTicks+ticks))
		SysTaskDelay(1);
}

void go(){
	char date[128];
	char time[128];
	char Ampm[128];
	char day[128];
	WinHandle wh;
	UInt16 i;
	UInt32 secs=0;
	BitmapPtr* b;
	LocalID* LIDs;
	UInt16 numBitmaps,currBitmap=0;
	Boolean exit=false;
	RectangleType rect;
	UInt32 secondsPerPic=START_SECS_PER_PIC;
	alarmInfoType alarm={false,0,0,NULL,0,""};
	UInt8 showTimeDate=3;
	
	RctSetRectangle(&rect,0,0,160,160);
	
	
	numBitmaps=enumerateBitmaps(&b,&LIDs);
	
	if(numBitmaps!=0){
	
		wh=WinCreateOffscreenWindow(160,160,screenFormat,&i);
		
		while(!exit){
			wh=WinSetDrawWindow(wh);
			
			if(TimGetSeconds()-secs>secondsPerPic){
				currBitmap=(currBitmap+1)%numBitmaps;
				secs=TimGetSeconds();
			}

			WinDrawBitmap(b[currBitmap],0,0);
			
			dateTimeFromSecs(TimGetSeconds(),date,time,Ampm,day);
			if(showTimeDate&1){
				drawChromeString(time,timeX,timeY);
				drawChromeString(Ampm,time2X,time2Y);
			}
			
			if(showTimeDate&2){
				DrawCoolString(date,StrLen(date),dateX,dateY);
				DrawCoolString(day,StrLen(day),dayX,dayY);
			}
			
			wh=WinSetDrawWindow(wh);
			WinDrawBitmap(WinGetBitmap(wh),0,0);
			SysTaskDelayNoKeys(20);
			
			if(KeyCurrentState()==0x00000008){
				waitKeyRelease();
				
				menu(&exit,&secondsPerPic,&numBitmaps,b,LIDs,&alarm);
				
			}
			if(KeyCurrentState()==0x00000010){
				waitKeyRelease();
				
				showTimeDate=(showTimeDate+1)%4;
				
			}
			if(KeyCurrentState()==0x00000020){
				waitKeyRelease();
				
				currBitmap=(currBitmap+numBitmaps-1)%numBitmaps;
				secs=TimGetSeconds();
				
			}
			if(KeyCurrentState()==0x00000040){
				waitKeyRelease();
				
				currBitmap=(currBitmap+1)%numBitmaps;
				secs=TimGetSeconds();
				
			}
			processCheckAlarm(&alarm);
			
		}
		WinDeleteWindow(wh,false);
	}
	
	releaseBitmaps(b,LIDs,numBitmaps);
}



void run(){
	UInt32 depth=16;
	UInt32 oldDepth;
	UInt8 font;
	UInt8 of;
	UInt8 ob;
	UInt8 ot;
	UInt32 mask;

	WinScreenMode(winScreenModeGet,NULL,NULL,&oldDepth,NULL);
	WinScreenMode(winScreenModeSet,NULL,NULL,&depth,NULL);
	of=WinSetForeColor(255);
	ob=WinSetBackColor(0);
	
	font=FntSetFont(7);
	ot=WinSetTextColor(255);
	mask=KeySetMask(0);
	go();
	KeySetMask(mask);
	WinScreenMode(winScreenModeSet,NULL,NULL,&oldDepth,NULL);
	WinSetBackColor(ob);
	WinSetForeColor(of);
	WinSetTextColor(ot);
	FntSetFont(font);
}

UInt32 PilotMain(UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
{
	if(cmd==0){
		run();
	}
	return errNone;
}
